package dev.pfaff.altargui.util.listenable;

import java.lang.invoke.MethodHandles;
import java.lang.invoke.VarHandle;
import java.util.Objects;
import java.util.function.Consumer;

public class ListenableValue<T> extends AbstractListenable implements ValueListenable<T> {
	private static final VarHandle VH_value;

	static {
		try {
			VH_value = MethodHandles.lookup().findVarHandle(ListenableValue.class, "value", Object.class);
		} catch (NoSuchFieldException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	private T value;

	public ListenableValue() {}

	public ListenableValue(T initialValue) {
		value = initialValue;
	}

	protected boolean equals(T a, T b) {
		return Objects.equals(a, b);
	}

	@Override
	public final ListenerDisposer addListener(Consumer<T> listener) {
		if (addListener((Object)listener)) {
			listener.accept(value);
		}
		return () -> removeListener(listener);
	}

	private void notifyUpdateValue(T value) {
		forEachListener(listener -> {
			switch (listener) {
				case Runnable runnable -> runnable.run();
				case Consumer consumer -> consumer.accept(value);
				default -> {}
			}
		});
	}

	public final T value() {
		return value;
	}

	public final void value(T value) {
		if (!equals(this.value, value)) {
			this.value = value;
			notifyUpdateValue(value);
		}
	}

	/**
	 * Identity is used instead of {@link #equals(Object, Object)}.
	 */
	public final boolean atomicCompareAndSet(T expected, T value) {
		if (VH_value.compareAndSet(this, expected, value)) {
			notifyUpdateValue(value);
			return true;
		}
		return false;
	}
}
