package dev.pfaff.altargui.util.listenable;

import java.util.function.Consumer;
import java.util.function.Function;

public final class MappingListenableValue<T, R> implements ValueListenable<R> {
	private final ValueListenable<T> upstream;
	private final Function<T, R> mapper;

	public MappingListenableValue(ValueListenable<T> upstream, Function<T, R> mapper) {
		this.upstream = upstream;
		this.mapper = mapper;
	}

	@Override
	public ListenerDisposer addListener(Runnable listener) {
		return upstream.addListener(listener);
	}

	@Override
	public ListenerDisposer addListener(Consumer<R> listener) {
		return upstream.addListener(t -> listener.accept(mapper.apply(t)));
	}

	@Override
	public <R1> ValueListenable<R1> map(Function<R, R1> mapper) {
		return new MappingListenableValue<>(upstream, this.mapper.andThen(mapper));
	}
}
