package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.element.SingleChildWidgetInstance;
import org.jetbrains.annotations.NotNull;

import java.util.function.Function;
import java.util.function.Supplier;

/**
 * The {@code supplier} will only be called when the widget is instantiated. It is not tracked. Thus, if you ever
 * need to refresh, you will need to key the widget with the whatever input should trigger a refresh.
 */
public record OnceWidget<T>(Supplier<T> supplier, Function<T, Widget> builder) implements Widget {
	@Override
	public @NotNull WidgetInstance<OnceWidget<T>> instantiate(WidgetInstantiationArgs args) {
		return new State<>(args);
	}

	private static final class State<T> extends SingleChildWidgetInstance<OnceWidget<T>> {
		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, OnceWidget<T> widget, boolean init) {
			if (init) {
				setChild(stateContext, widget.builder.apply(widget.supplier.get()));
			}
		}
	}
}
